@@ -1,11 +1,15 @@
 mod app_state;
 mod cmd_result;
+mod content_skin;
 mod content_view;
 mod drawer_state;
-mod entry_state;
+mod drawer_edit_state;
+mod drawer_focus;
 mod global_view;
 mod keys;
+mod matched_string;
 mod password_input_state;
+mod search_state;
 mod scroll;
 mod status_view;
 mod title_view;
@@ -13,7 +17,10 @@ mod app;
 mod view;
 
 use {
-    crate::{core::Closet, error::SafeClosetError},
+    crate::{
+        core::Closet,
+        error::SafeClosetError,
+    },
     crossterm::{
         self, cursor,
         terminal::{EnterAlternateScreen, LeaveAlternateScreen},
@@ -25,12 +32,16 @@ use {
 pub(crate) use {
     app_state::*,
     cmd_result::*,
+    content_skin::*,
     content_view::*,
     drawer_state::*,
-    entry_state::*,
+    drawer_edit_state::*,
+    drawer_focus::*,
     global_view::*,
     keys::*,
+    matched_string::*,
     password_input_state::*,
+    search_state::*,
     scroll::*,
     status_view::*,
     title_view::*,
